package org.jastadd.ragconnect.tests;

import java.io.File;
import java.io.IOException;
import java.nio.charset.Charset;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;

import static org.junit.jupiter.api.Assertions.fail;

/**
 * Utility methods for tests.
 *
 * @author rschoene - Initial contribution
 */
public class TestUtils {

  public static final double DELTA = 0.001d;

  public static String getMqttHost() {
    if (System.getenv("GITLAB_CI") != null) {
      // we are in the CI, so use "mqtt" as host
      return "mqtt";
    } {
      // else assume a locally running mqtt broker
      return "localhost";
    }
  }

  public static String mqttUri(String path) {
    return "mqtt://" + getMqttHost() + "/" + path;
  }

  public static String restUri(String path, int port) {
    return "rest://localhost:" + port + "/" + path;
  }

  public static int getMqttDefaultPort() {
    return 1883;
  }

  public static int exec(Class<?> klass, String[] args, File err) throws IOException,
      InterruptedException {
    String javaHome = System.getProperty("java.home");
    String javaBin = javaHome + File.separator + "bin" + File.separator + "java";
    String classpath = System.getProperty("java.class.path");
    String className = klass.getName();

    String[] newArgs = new String[args.length + 4];
    newArgs[0] = javaBin;
    newArgs[1] = "-cp";
    newArgs[2] = classpath;
    newArgs[3] = className;
    System.arraycopy(args, 0, newArgs, 4, args.length);

    ProcessBuilder builder = new ProcessBuilder(newArgs);
//    builder.redirectOutput(err);
    builder.redirectError(err);

    Process process = builder.start();
    process.waitFor();
    return process.exitValue();
  }

  public static void testJaddContainReferenceToJackson(Path path, boolean shouldContain) {
    try {
      String content = Files.readString(path);
      boolean actualContain = content.contains("com.fasterxml.jackson.databind.ObjectMapper");
      if (actualContain && !shouldContain) {
        fail(path + " should not depend on jackson library, but does");
      }
      if (!actualContain && shouldContain) {
        fail(path + " does not depend on jackson library");
      }
    } catch (IOException e) {
      fail(e);
    }
  }

  public static String readFile(String path, Charset encoding)
      throws IOException {
    byte[] encoded = Files.readAllBytes(Paths.get(path));
    return new String(encoded, encoding);
  }

  static void waitForMqtt() throws InterruptedException {
    TimeUnit.SECONDS.sleep(2);
  }

  @SuppressWarnings({"unused", "rawtypes"})
  static class DefaultMappings {
    static class ReadNode extends defaultOnlyRead.ast.ASTNode {
      public static boolean _apply__DefaultBytesToBooleanMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToBooleanMapping(input);
      }
      public static int _apply__DefaultBytesToIntMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToIntMapping(input);
      }
      public static short _apply__DefaultBytesToShortMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToShortMapping(input);
      }
      public static long _apply__DefaultBytesToLongMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToLongMapping(input);
      }
      public static float _apply__DefaultBytesToFloatMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToFloatMapping(input);
      }
      public static double _apply__DefaultBytesToDoubleMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToDoubleMapping(input);
      }
      public static char _apply__DefaultBytesToCharMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToCharMapping(input);
      }
      public static String _apply__DefaultBytesToStringMapping(byte[] input) throws Exception {
        return defaultOnlyRead.ast.ASTNode._apply__DefaultBytesToStringMapping(input);
      }
    }

    static class WriteNode extends defaultOnlyWrite.ast.ASTNode {
      public static byte[] _apply__DefaultBooleanToBytesMapping(boolean input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultBooleanToBytesMapping(input);
      }
      public static byte[] _apply__DefaultIntToBytesMapping(int input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultIntToBytesMapping(input);
      }
      public static byte[] _apply__DefaultShortToBytesMapping(short input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultShortToBytesMapping(input);
      }
      public static byte[] _apply__DefaultLongToBytesMapping(long input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultLongToBytesMapping(input);
      }
      public static byte[] _apply__DefaultFloatToBytesMapping(float input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultFloatToBytesMapping(input);
      }
      public static byte[] _apply__DefaultDoubleToBytesMapping(double input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultDoubleToBytesMapping(input);
      }
      public static byte[] _apply__DefaultCharToBytesMapping(char input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultCharToBytesMapping(input);
      }
      public static byte[] _apply__DefaultStringToBytesMapping(String input) throws Exception {
        return defaultOnlyWrite.ast.ASTNode._apply__DefaultStringToBytesMapping(input);
      }
    }

    public static boolean BytesToBool(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToBooleanMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return false;
      }
    }
    public static int BytesToInt(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToIntMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static short BytesToShort(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToShortMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static long BytesToLong(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToLongMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static float BytesToFloat(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToFloatMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static double BytesToDouble(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToDoubleMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static char BytesToChar(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToCharMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return 0;
      }
    }
    public static String BytesToString(byte[] input) {
      try {
        return ReadNode._apply__DefaultBytesToStringMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] BoolToBytes(boolean input) {
      try {
        return WriteNode._apply__DefaultBooleanToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] IntToBytes(int input) {
      try {
        return WriteNode._apply__DefaultIntToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] ShortToBytes(short input) {
      try {
        return WriteNode._apply__DefaultShortToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] LongToBytes(long input) {
      try {
        return WriteNode._apply__DefaultLongToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] FloatToBytes(float input) {
      try {
        return WriteNode._apply__DefaultFloatToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] DoubleToBytes(double input) {
      try {
        return WriteNode._apply__DefaultDoubleToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] CharToBytes(char input) {
      try {
        return WriteNode._apply__DefaultCharToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
    public static byte[] StringToBytes(String input) {
      try {
        return WriteNode._apply__DefaultStringToBytesMapping(input);
      } catch (Exception e) {
        e.printStackTrace();
        return null;
      }
    }
  }
}
